# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

""" CVE Binary Tool tests for the extractor function """
import lzma
import os
import shutil
import tarfile
import tempfile
import unittest
import unittest.mock
from io import BytesIO
from test.utils import CURL_7_20_0_URL, TMUX_DEB, download_file, event_loop
from zipfile import ZipFile, ZipInfo

import pytest

from cve_bin_tool.extractor import Extractor
from cve_bin_tool.util import inpath

# Enable logging if tests are not passing to help you find errors
# import logging
# logging.basicConfig(level=logging.DEBUG)

CAB_TEST_FILE_PATH = os.path.join(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "assets"),
    "cab-test-python3.8.cab",
)
BAD_EXE_FILE = os.path.join(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "assets"),
    "empty-file.exe",
)
BAD_ZIP_FILE = os.path.join(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "assets"),
    "empty-file.zip",
)


class TestExtractorBase:
    """Test methods for extraction of various file types"""

    @classmethod
    def setup_class(cls):
        cls.extractor = Extractor()
        cls.tempdir = tempfile.mkdtemp(prefix="cve-bin-tool-")

    @classmethod
    def teardown_class(cls):
        shutil.rmtree(cls.tempdir)

    @pytest.mark.asyncio
    async def extract_files(self, filenames):
        """Make sure all test files are present"""
        for filename in filenames:
            assert os.path.exists(
                os.path.join(self.tempdir, filename)
            ), f"test file {filename} was not found"
        # Make sure extraction reports success
        for filename in filenames:
            async with self.extractor as ectx:
                yield await ectx.aio_extract(os.path.join(self.tempdir, filename))


class TestExtractor(TestExtractorBase):
    """Test methods for the extractor functionality"""

    def test_can_extract(self):
        """Test that the can_extract function knows what it can do"""
        assert self.extractor.can_extract(".tar.bz2")
        assert self.extractor.can_extract(".zip")
        assert self.extractor.can_extract(".deb")


class TestExtractFileTar(TestExtractorBase):
    """Tetss for tar file extraction"""

    def setup_method(self):
        for filename, tarmode in [
            ("test.tgz", "w:gz"),
            ("test.tar.gz", "w:gz"),
            ("test.tar.bz2", "w:bz2"),
            ("test.tar", "w"),
        ]:
            tarpath = os.path.join(self.tempdir, filename)
            tar = tarfile.open(tarpath, mode=tarmode)
            data = b"feedface"
            addfile = BytesIO(data)
            info = tarfile.TarInfo(name="test.txt")
            info.size = len(data)
            tar.addfile(tarinfo=info, fileobj=addfile)
            tar.close()
        tarpath = os.path.join(self.tempdir, "test.tar")
        tarpath_xz = os.path.join(self.tempdir, "test.tar.xz")
        with open(tarpath, "rb") as infile, lzma.open(tarpath_xz, "w") as outfile:
            outfile.write(infile.read())

    @pytest.mark.asyncio
    async def test_extract_file_tar(self):
        """Test the tar file extraction"""
        async for path in self.extract_files(
            [
                "test" + e
                for e in self.extractor.file_extractors[self.extractor.extract_file_tar]
            ]
        ):
            assert os.path.isdir(path)

    @pytest.mark.asyncio
    async def test_extract_cleanup(self):
        """Make sure tar extractor cleans up after itself"""
        async with self.extractor as ectx:
            extracted_path = await ectx.aio_extract(
                os.path.join(self.tempdir, "test.tar")
            )
            assert os.path.isdir(extracted_path)
        assert not os.path.exists(extracted_path)


class TestExtractFileRpm(TestExtractorBase):
    """Tests for the rpm file extractor"""

    @classmethod
    def setup_class(cls):
        super().setup_class()
        download_file(CURL_7_20_0_URL, os.path.join(cls.tempdir, "test.rpm"))

    @pytest.mark.asyncio
    async def test_extract_file_rpm(self):
        """Test the rpm file extraction"""
        async for extracted_path in self.extract_files(
            [
                "test" + e
                for e in self.extractor.file_extractors[self.extractor.extract_file_rpm]
            ]
        ):
            assert os.path.isfile(os.path.join(extracted_path, "usr", "bin", "curl"))

    @pytest.mark.asyncio
    async def test_extract_file_rpm_no_rpm2cipo(self):
        """Test rpm extraction using rpmfile"""
        with unittest.mock.patch(
            "cve_bin_tool.async_utils.aio_inpath",
            return_value=False,
        ), unittest.mock.patch(
            "cve_bin_tool.async_utils.aio_run_command",
        ) as mock_aio_run_command:
            await self.test_extract_file_rpm()
            mock_aio_run_command.assert_not_called()


class TestExtractFileDeb(TestExtractorBase):
    """Tests for deb file extractor"""

    def setup_method(self):
        assert inpath("ar"), "Required tool 'ar' not found"
        download_file(TMUX_DEB, os.path.join(self.tempdir, "test.deb"))
        shutil.copyfile(
            os.path.join(self.tempdir, "test.deb"),
            os.path.join(self.tempdir, "test.ipk"),
        )

    @pytest.mark.asyncio
    async def test_extract_file_deb(self):
        """Test the deb file extraction"""
        async for extracted_path in self.extract_files(
            [
                "test" + e
                for e in self.extractor.file_extractors[self.extractor.extract_file_deb]
            ]
        ):
            assert os.path.isfile(os.path.join(extracted_path, "usr", "bin", "tmux"))


class TestExtractFileCab(TestExtractorBase):
    """Tests for the cab file extractor"""

    def setup_method(self):
        shutil.copyfile(CAB_TEST_FILE_PATH, os.path.join(self.tempdir, "test.cab"))

    @pytest.mark.asyncio
    async def test_extract_file_cab(self):
        """Test the cab file extraction"""
        async for extracted_path in self.extract_files(
            [
                "test" + e
                for e in self.extractor.file_extractors[self.extractor.extract_file_cab]
            ]
        ):

            assert os.path.isfile(
                os.path.join(
                    os.path.join(os.path.join(extracted_path, "usr"), "bin"),
                    "python3.8",
                )
            )


class TestExtractFileZip(TestExtractorBase):
    """Tests for the zip file extractor
    This extractor also handles jar, apk, and sometimes exe files
    when the exe is a self-extracting zipfile"""

    def setup_method(self):
        for filename in [
            "test.exe",
            "test.zip",
            "test.jar",
            "test.apk",
            "test.msi",
            "test.egg",
            "test.whl",
        ]:
            zippath = os.path.join(self.tempdir, filename)
            with ZipFile(zippath, "w") as zipfile:
                zipfile.writestr(ZipInfo("test.txt"), "feedface")

    @pytest.mark.asyncio
    async def test_extract_file_zip(self):
        """Test the zip file extraction"""
        async for path in self.extract_files(
            [
                "test" + e
                for e in self.extractor.file_extractors[self.extractor.extract_file_zip]
            ]
        ):
            assert os.path.isdir(path)

    @pytest.mark.asyncio
    async def test_bad_zip(self):
        """Test handling of invalid zip files.  No errors should be raised.
        Log messages differ for .exe and .zip and are tested in test_cli.py
        """

        self.extract_files(BAD_EXE_FILE)
        self.extract_files(BAD_ZIP_FILE)



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for wpa_supplicant

https://www.cvedetails.com/product/29296/W1.fi-Wpa-Supplicant.html?vendor_id=12005

"""
from cve_bin_tool.checkers import Checker


class WpaSupplicantChecker(Checker):
    CONTAINS_PATTERNS = [
        r"wpa_supplicant couldn't remove this interface",
        r"wpa_supplicant knows nothing about this interface",
    ]
    FILENAME_PATTERNS = [r"wpa_supplicant"]
    VERSION_PATTERNS = [r"wpa_supplicant v([0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("w1.fi", "wpa_supplicant")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for bash

https://www.cvedetails.com/vulnerability-list/vendor_id-72/product_id-21050/GNU-Bash.html

"""
from cve_bin_tool.checkers import Checker


class BashChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"bash"]
    VERSION_PATTERNS = [r"Bash version ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("gnu", "bash")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for curl CLI

References:
https://curl.haxx.se/docs/security.html
http://www.cvedetails.com/vulnerability-list/vendor_id-12682/Haxx.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=12682&product_id=0&version_id=0&orderby=3&cvssscoremin=0

Note: Some of the "first vulnerable in" data may not be entered correctly.
"""
from cve_bin_tool.checkers import Checker


class CurlChecker(Checker):
    # FIXME: fix contains pattern
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"curl"]
    VERSION_PATTERNS = [r"curl ([678]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("haxx", "curl")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "openswan", "version": "2.6.30", "version_strings": ["Openswan 2.6.30"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/openswan/2.6.31/1.fc14/x86_64/",
        "package_name": "openswan-2.6.31-1.fc14.x86_64.rpm",
        "product": "openswan",
        "version": "2.6.31",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for polarssl

This checker currently works on only fedora distribution, because of lack of common signatures
in other distributions, with unsuccessful attempts made for CentOS and ubuntu distributions.

https://www.cvedetails.com/product/22470/Polarssl-Polarssl.html?vendor_id=12001

"""
from cve_bin_tool.checkers import Checker


class PolarsslFedoraChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"libpolarssl.so."]
    VERSION_PATTERNS = [
        r"libpolarssl.so.([0-9]+\.[0-9]+\.[0-9]+)"
    ]  # patterns like this aren't ideal
    VENDOR_PRODUCT = [("polarssl", "polarssl")]


"""
Using filenames (containing patterns like '.so' etc.) in the binaries as VERSION_PATTERNS aren't ideal.
The reason behind this is that these might depend on who packages the file (like it 
might work on fedora but not on ubuntu)
"""



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for Ncurses

References:
https://www.cvedetails.com/vulnerability-list/vendor_id-72/product_id-38464/GNU-Ncurses.html

RSS feed: http://www.cvedetails.com/vulnerability-feed.php?vendor_id=72&product_id=38464&version_id=0&orderby=3&cvssscoremin=0

"""
from cve_bin_tool.checkers import Checker


class NcursesChecker(Checker):
    CONTAINS_PATTERNS = [r"_nc_first_name", r"_nc_infotocap", r"if NCURSES_XNAMES"]
    FILENAME_PATTERNS = [
        r"libcurses",
        r"libform",
        r"libmenu",
        r"libncurses",
        r"libncurses\+{2}",
        r"libpanel",
        r"libcursesw",
        r"libformw",
        r"libmenuw",
        r"libncursesw",
        r"libncurses\+{2}w",
        r"libpanelw",
    ]
    VERSION_PATTERNS = [r"ncurses ([0-9]+\.[0-9]+)", r"infocmp-([0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("gnu", "ncurses")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import subprocess
from os import environ
from os.path import dirname, join
from sys import platform

import distro
import pytest

from cve_bin_tool.error_handler import ErrorMode
from cve_bin_tool.package_list_parser import (
    SUPPORTED_DISTROS,
    EmptyTxtError,
    InvalidListError,
    PackageListParser,
    Remarks,
)
from cve_bin_tool.util import ProductInfo


class TestPackageListParser:
    TXT_PATH = join(dirname(__file__), "txt")

    REQ_PARSED_TRIAGE_DATA = {
        ProductInfo(vendor="httplib2_project*", product="httplib2", version="0.18.1"): {
            "default": {"remarks": Remarks.Unexplored, "comments": "", "severity": ""},
            "paths": {""},
        },
        ProductInfo(vendor="python*", product="requests", version="2.25.1"): {
            "default": {"remarks": Remarks.Unexplored, "comments": "", "severity": ""},
            "paths": {""},
        },
        ProductInfo(vendor="html5lib*", product="html5lib", version="0.99"): {
            "default": {"remarks": Remarks.Unexplored, "comments": "", "severity": ""},
            "paths": {""},
        },
    }

    UBUNTU_PARSED_TRIAGE_DATA = {
        ProductInfo(vendor="gnu*", product="bash", version="5.0-6ubuntu1.1"): {
            "default": {"remarks": Remarks.Unexplored, "comments": "", "severity": ""},
            "paths": {""},
        },
        ProductInfo(vendor="gnu*", product="binutils", version="2.34-6ubuntu1.1"): {
            "default": {"remarks": Remarks.Unexplored, "comments": "", "severity": ""},
            "paths": {""},
        },
        ProductInfo(vendor="gnu*", product="wget", version="1.20.3-1ubuntu1"): {
            "default": {"remarks": Remarks.Unexplored, "comments": "", "severity": ""},
            "paths": {""},
        },
    }

    @pytest.mark.parametrize("filepath", [join(TXT_PATH, "nonexistent.txt")])
    def test_nonexistent_txt(self, filepath):
        package_list = PackageListParser(filepath, error_mode=ErrorMode.FullTrace)
        with pytest.raises(FileNotFoundError):
            package_list.parse_list()

    @pytest.mark.parametrize(
        "filepath, exception", [(join(TXT_PATH, "empty.txt"), EmptyTxtError)]
    )
    def test_empty_txt(self, filepath, exception):
        package_list = PackageListParser(filepath, error_mode=ErrorMode.FullTrace)
        with pytest.raises(exception):
            package_list.parse_list()

    @pytest.mark.parametrize(
        "filepath, exception", [(join(TXT_PATH, "not_txt.csv"), InvalidListError)]
    )
    def test_not_txt(self, filepath, exception):
        package_list = PackageListParser(filepath, error_mode=ErrorMode.FullTrace)
        with pytest.raises(exception):
            package_list.parse_list()

    @pytest.mark.parametrize(
        "filepath, parsed_data",
        [(join(TXT_PATH, "test_requirements.txt"), REQ_PARSED_TRIAGE_DATA)],
    )
    def test_valid_requirements(self, filepath, parsed_data):
        # packages is installed from test_requirements with specific versions for the test to pass
        subprocess.run(["pip", "install", "-r", filepath])
        package_list = PackageListParser(filepath, error_mode=ErrorMode.FullTrace)
        assert package_list.parse_list() == parsed_data
        # Update the packages back to latest
        subprocess.run(["pip", "install", "httplib2", "requests", "html5lib", "-U"])

    @pytest.mark.skipif(
        distro.id() not in SUPPORTED_DISTROS,
        reason=f"Test for {','.join(SUPPORTED_DISTROS)} systems",
    )
    @pytest.mark.parametrize(
        "filepath, exception",
        [(join(TXT_PATH, "test_broken_linux_list.txt"), InvalidListError)],
    )
    def test_invalid_linux_list(self, filepath, exception):
        package_list = PackageListParser(filepath, error_mode=ErrorMode.FullTrace)
        with pytest.raises(exception):
            package_list.parse_list()

    @pytest.mark.skipif(
        "ACTIONS" not in environ
        or not platform == "linux"
        or ("ubuntu" not in distro.id() and "20.04" not in distro.version()),
        reason="Running locally requires root permission",
    )
    @pytest.mark.parametrize(
        "filepath, parsed_data",
        [(join(TXT_PATH, "test_ubuntu_list.txt"), UBUNTU_PARSED_TRIAGE_DATA)],
    )
    def test_valid_ubuntu_list(self, filepath, parsed_data):
        subprocess.run(
            [
                "sudo",
                "apt-get",
                "install",
                "bash=5.0-6ubuntu1.1",
                "binutils=2.34-6ubuntu1.1",
                "wget=1.20.3-1ubuntu1",
            ]
        )
        package_list = PackageListParser(filepath, error_mode=ErrorMode.FullTrace)
        assert package_list.parse_list() == parsed_data



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for ffmpeg

References:
https://www.ffmpeg.org/
https://www.cvedetails.com/vulnerability-list/vendor_id-3611/Ffmpeg.html

Note: Some of the "first vulnerable in" data may not be entered correctly.
"""
from cve_bin_tool.checkers import Checker


class FfmpegChecker(Checker):
    # NOTE:XXX: can we optimize version pattern
    CONTAINS_PATTERNS = [
        r"Codec '%s' is not recognized by FFmpeg.",
        r"Codec '%s' is known to FFmpeg, but no %s for it are available.",
    ]
    FILENAME_PATTERNS = [r"ffmpeg"]
    VERSION_PATTERNS = [r"%s version ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("ffmpeg", "ffmpeg")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "gnutls", "version": "2.1.6", "version_strings": ["gnutls-cli 2.1.6"]},
    {
        "product": "gnutls",
        "version": "2.3.11",
        "version_strings": ["gnutls-serv 2.3.11"],
    },
]
package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "gnutls-utils-3.3.29-9.el7_6.x86_64.rpm",
        "product": "gnutls",
        "version": "3.3.29",
    },
    {
        "url": "http://archive.ubuntu.com/ubuntu/pool/universe/g/gnutls28/",
        "package_name": "gnutls-bin_3.4.10-4ubuntu1.7_amd64.deb",
        "product": "gnutls",
        "version": "3.4.10",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import os
import re

import pytest

from cve_bin_tool.error_handler import ErrorMode
from cve_bin_tool.input_engine import (
    InputEngine,
    InvalidCsvError,
    InvalidJsonError,
    MissingFieldsError,
    Remarks,
)
from cve_bin_tool.util import ProductInfo


class TestInputEngine:
    CSV_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), "csv")
    JSON_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), "json")
    PARSED_TRIAGE_DATA = {
        ProductInfo("haxx", "curl", "7.59.0"): {
            "default": {"comments": "", "remarks": Remarks.NewFound, "severity": ""},
            "paths": {""},
        },
        ProductInfo("haxx", "libcurl", "7.59.0"): {
            "default": {"comments": "", "remarks": Remarks.Unexplored, "severity": ""},
            "paths": {""},
        },
        ProductInfo("libjpeg-turbo", "libjpeg-turbo", "2.0.1"): {
            "CVE-2018-19664": {
                "comments": "High priority need to resolve fast",
                "remarks": Remarks.Confirmed,
                "severity": "CRITICAL",
            },
            "default": {
                "comments": "Need to mitigate cves of this product",
                "remarks": Remarks.Unexplored,
                "severity": "HIGH",
            },
            "paths": {""},
        },
        ProductInfo("mit", "kerberos", "1.15.1"): {
            "default": {"comments": "", "remarks": Remarks.Unexplored, "severity": ""},
            "paths": {""},
        },
        ProductInfo("mit", "kerberos_5", "5-1.15.1"): {
            "default": {"comments": "", "remarks": Remarks.Confirmed, "severity": ""},
            "paths": {""},
        },
        ProductInfo("ssh", "ssh2", "2.0"): {
            "default": {"comments": "", "remarks": Remarks.Mitigated, "severity": ""},
            "paths": {""},
        },
        ProductInfo("sun", "sunos", "5.4"): {
            "default": {"comments": "", "remarks": Remarks.Mitigated, "severity": ""},
            "paths": {""},
        },
    }
    MISSING_FIELD_REGEX = re.compile(
        r"({[' ,](([a-z])+[' ,]{1,4})+}) are required fields"
    )

    @pytest.mark.parametrize(
        "filepath",
        (
            os.path.join(CSV_PATH, "nonexistent.csv"),
            os.path.join(JSON_PATH, "nonexistent.json"),
        ),
    )
    def test_nonexistent_file(self, filepath):
        input_engine = InputEngine(filepath, error_mode=ErrorMode.FullTrace)
        with pytest.raises(FileNotFoundError):
            input_engine.parse_input()

    @pytest.mark.parametrize(
        "filepath, exception",
        (
            (os.path.join(CSV_PATH, "bad.csv"), InvalidCsvError),
            (os.path.join(JSON_PATH, "bad.json"), InvalidJsonError),
        ),
    )
    def test_invalid_file(self, filepath, exception):
        input_engine = InputEngine(filepath, error_mode=ErrorMode.FullTrace)
        with pytest.raises(exception):
            input_engine.parse_input()

    @pytest.mark.parametrize(
        "filepath, missing_fields",
        (
            (os.path.join(CSV_PATH, "bad_product.csv"), {"product"}),
            (
                os.path.join(CSV_PATH, "bad_heading.csv"),
                {"vendor", "product", "version"},
            ),
            (
                os.path.join(JSON_PATH, "bad_heading.json"),
                {"vendor", "product", "version"},
            ),
        ),
    )
    def test_missing_fields(self, filepath, missing_fields):
        input_engine = InputEngine(filepath, error_mode=ErrorMode.FullTrace)
        with pytest.raises(MissingFieldsError) as exc:
            input_engine.parse_input()

        match = self.MISSING_FIELD_REGEX.search(exc.value.args[0])
        raised_fields = match.group(1)

        assert missing_fields - eval(raised_fields) == set()

    @pytest.mark.parametrize(
        "filepath, parsed_data",
        (
            (os.path.join(CSV_PATH, "test_triage.csv"), PARSED_TRIAGE_DATA),
            (os.path.join(JSON_PATH, "test_triage.json"), PARSED_TRIAGE_DATA),
        ),
    )
    def test_valid_file(self, filepath, parsed_data):
        input_engine = InputEngine(filepath, error_mode=ErrorMode.FullTrace)
        assert dict(input_engine.parse_input()) == parsed_data



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "openjpeg", "version": "1.5.1", "version_strings": ["openjpeg-1.5.1"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/openjpeg/1.5.0/5.fc18/x86_64/",
        "package_name": "openjpeg-1.5.0-5.fc18.x86_64.rpm",
        "product": "openjpeg",
        "version": "1.5.0",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "openjpeg-1.5.1-18.el7.x86_64.rpm",
        "product": "openjpeg",
        "version": "1.5.1",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
To determine if a file is executable we read the first few bytes of a file and
check it against various signatures for different executable file formats.
"""
import inspect
import sys

from cve_bin_tool.async_utils import FileIO, run_coroutine


async def aio_is_binary(filename):
    """Read the magic bytes from a file and determine if it is an executable
    binary."""
    signature = await read_signature(filename)
    for name, method in inspect.getmembers(
        sys.modules[__name__], predicate=inspect.isfunction
    ):
        if name.startswith("check_"):
            if method(filename, signature):
                return True
    return False


def is_binary(filename):
    return run_coroutine(aio_is_binary(filename))


async def read_signature(filename, length=4):
    """Read the signature, first length bytes, from filename."""
    async with FileIO(filename, "rb") as file_handle:
        return await file_handle.read(length)


def check_elf(_filename, signature):
    """Check for an ELF signature."""
    return signature == b"\x7f\x45\x4c\x46"


def check_pe(_filename, signature):
    """Check for windows/dos PE signature, aka 0x5a4d."""
    return signature[:4] == b"\x4d\x5a"


def check_fake_test(_filename, signature):
    """check for fake tests under windows."""
    return signature == b"MZ\x90\x00"



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "memcached", "version": "1.4.1", "version_strings": ["memcached 1.4.1"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/memcached/1.4.0/1.fc12/x86_64/",
        "package_name": "memcached-1.4.0-1.fc12.x86_64.rpm",
        "product": "memcached",
        "version": "1.4.0",
    },
    {
        "url": "https://mirrors.edge.kernel.org/centos/7/os/x86_64/Packages/",
        "package_name": "memcached-1.4.15-10.el7_3.1.x86_64.rpm",
        "product": "memcached",
        "version": "1.4.15",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "libarchive",
        "version": "3.2.0",
        "version_strings": ["libarchive 3.2.0"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/libarchive/3.3.3/1.fc28/x86_64/",
        "package_name": "libarchive-3.3.3-1.fc28.x86_64.rpm",
        "product": "libarchive",
        "version": "3.3.3",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "libarchive-3.1.2-14.el7_7.x86_64.rpm",
        "product": "libarchive",
        "version": "3.1.2",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

""" CVE Checkers """
import collections
import re

from cve_bin_tool.error_handler import InvalidCheckerError
from cve_bin_tool.util import regex_find

__all__ = [
    "Checker",
    "VendorProductPair",
    "accountsservice",
    "avahi",
    "bash",
    "bind",
    "binutils",
    "bolt",
    "bubblewrap",
    "busybox",
    "bzip2",
    "cryptsetup",
    "cups",
    "curl",
    "dnsmasq",
    "dovecot",
    "dpkg",
    "enscript",
    "expat",
    "ffmpeg",
    "freeradius",
    "ftp",
    "gcc",
    "gimp",
    "gnupg",
    "gnutls",
    "glibc",
    "gpgme",
    "gstreamer",
    "haproxy",
    "hostapd",
    "icecast",
    "icu",
    "irssi",
    "kerberos",
    "libarchive",
    "libdb",
    "libgcrypt",
    "libjpeg",
    "libnss",
    "libsndfile",
    "libsoup",
    "libssh2",
    "libtiff",
    "libvirt",
    "libxslt",
    "lighttpd",
    "logrotate",
    "mariadb",
    "memcached",
    "mysql",
    "ncurses",
    "nessus",
    "netpbm",
    "nginx",
    "node",
    "ntp",
    "openafs",
    "openjpeg",
    "openldap",
    "openssh",
    "openssl",
    "openswan",
    "openvpn",
    "p7zip",
    "png",
    "polarssl_fedora",
    "postgresql",
    "pspp",
    "python",
    "qt",
    "radare2",
    "rsyslog",
    "samba",
    "sqlite",
    "strongswan",
    "subversion",
    "syslogng",
    "systemd",
    "tcpdump",
    "varnish",
    "webkitgtk",
    "wireshark",
    "wpa_supplicant",
    "xerces",
    "xml2",
    "zlib",
]

VendorProductPair = collections.namedtuple("VendorProductPair", ["vendor", "product"])


class CheckerMetaClass(type):
    def __init__(cls, name, bases, namespace, **kwargs):
        """
        Needed for compatibility with Python 3.5
        """
        super().__init__(name, bases, namespace)

    def __new__(cls, name, bases, props):
        # Create the class
        cls = super().__new__(cls, name, bases, props)
        # HACK Skip validation if this class is the base class
        if name == "Checker":
            return cls
        # Validate that we have at least one vendor product pair
        if len(cls.VENDOR_PRODUCT) < 1:
            raise AssertionError(f"{name} needs at least one vendor product pair")
        # Validate that each vendor product pair is of length 2
        cls.VENDOR_PRODUCT = list(
            map(lambda vpkg: VendorProductPair(*vpkg), cls.VENDOR_PRODUCT)
        )
        # Validate that vendor product pair is in lowercase
        for items in cls.VENDOR_PRODUCT:
            for vp in items:
                if vp.islower() != True:
                    raise InvalidCheckerError(
                        f"Checker {name} has a VENDOR_PRODUCT string that is not lowercase"
                    )
        # Compile regex
        cls.CONTAINS_PATTERNS = list(map(re.compile, cls.CONTAINS_PATTERNS))
        cls.VERSION_PATTERNS = list(map(re.compile, cls.VERSION_PATTERNS))
        cls.FILENAME_PATTERNS = list(map(re.compile, cls.FILENAME_PATTERNS))
        cls.CONTAINS_PATTERNS.extend(cls.VERSION_PATTERNS)
        # Return the new checker class
        return cls


class Checker(metaclass=CheckerMetaClass):
    CONTAINS_PATTERNS = []
    VERSION_PATTERNS = []
    FILENAME_PATTERNS = []
    VENDOR_PRODUCT = []

    def guess_contains(self, lines):
        for line in lines:
            if any(pattern.search(line) for pattern in self.CONTAINS_PATTERNS):
                return True
        return False

    def get_version(self, lines, filename):
        version_info = dict()

        if any(pattern.search(filename) for pattern in self.FILENAME_PATTERNS):
            version_info["is_or_contains"] = "is"

        if "is_or_contains" not in version_info and self.guess_contains(lines):
            version_info["is_or_contains"] = "contains"

        if "is_or_contains" in version_info:
            version_info["version"] = regex_find(lines, self.VERSION_PATTERNS)

        return version_info



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for gcc

https://www.cvedetails.com/vulnerability-list/vendor_id-72/product_id-960/GNU-GCC.html

"""
from cve_bin_tool.checkers import Checker


class GccChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTTERN = [r"gcc"]
    VERSION_PATTERNS = [
        r"gcc ([0-9]+\.[0-9]+\.[0-9]+)",
        r"gcc ([0-9]+\.[0-9]+)",
    ]
    VENDOR_PRODUCT = [("gnu", "gcc"), ("gcc", "gcc")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "varnish", "version": "4.1.1", "version_strings": ["varnish-4.1.1"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/varnish/4.0.5/1.el7/x86_64/",
        "package_name": "varnish-4.0.5-1.el7.x86_64.rpm",
        "product": "varnish",
        "version": "4.0.5",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "libsndfile",
        "version": "1.0.25",
        "version_strings": ["libsndfile-1.0.25"],
    }
]
package_test_data = [
    {
        "url": "http://mirror.centos.org/altarch/7/os/aarch64/Packages/",
        "package_name": "libsndfile-1.0.25-12.el7.aarch64.rpm",
        "product": "libsndfile",
        "version": "1.0.25",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "radare2", "version": "3.3.0", "version_strings": ["rafind2 v3.3.0"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/radare2/3.0.1/2.fc27/x86_64/",
        "package_name": "radare2-3.0.1-2.fc27.x86_64.rpm",
        "product": "radare2",
        "version": "3.0.1",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for bind

https://www.cvedetails.com/product/144/ISC-Bind.html?vendor_id=64

"""
from cve_bin_tool.checkers import Checker


class BindChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"named"]
    VERSION_PATTERNS = [r"BIND ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("isc", "bind")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "strongswan",
        "version": "4.6.3",
        "version_strings": ["strongSwan 4.6.3"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/strongswan/4.6.2/1.fc16/x86_64/",
        "package_name": "strongswan-4.6.2-1.fc16.x86_64.rpm",
        "product": "strongswan",
        "version": "4.6.2",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE checker for libssh2

https://www.cvedetails.com/product/31293/Libssh2-Libssh2.html?vendor_id=15300

"""
from cve_bin_tool.checkers import Checker


class Libssh2Checker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"libssh2"]
    VERSION_PATTERNS = [r"SSH-2.0-libssh2_([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("libssh2", "libssh2")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = {}
package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "krb5-libs-1.15.1-50.el7.x86_64.rpm",
        "product": "kerberos_5",
        "version": "1.15.1",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "nessus",
        "version": "6.8.1",
        "version_strings": ["nessus_lib_version", "Nessus 6.8.1"],
    }
]
package_test_data = [
    {
        "url": "https://archives.fedoraproject.org/pub/archive/fedora/linux/releases/30/Everything/aarch64/os/Packages/n/",
        "package_name": "nessus-libraries-2.2.11-16.fc29.aarch64.rpm",
        "product": "nessus",
        "version": "2.2.11",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE checker for ntp

References:
https://www.cvedetails.com/product/3682/?q=NTP

"""
from cve_bin_tool.checkers import Checker


class NtpChecker(Checker):
    CONTAINS_PATTERNS = [r"With high-traffic NTP servers, this can occur if the"]
    FILENAME_PATTERNS = [r"ntp"]
    VERSION_PATTERNS = [
        r"ntpd ([0-9]+\.[0-9]+\.[0-9]+p[0-9]+)",
        r"ntpdc ([0-9]+\.[0-9]+\.[0-9]+p[0-9]+)",
        r"ntpq ([0-9]+\.[0-9]+\.[0-9]+p[0-9]+)",
    ]
    VENDOR_PRODUCT = [("ntp", "ntp")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import ast
import os

from setuptools import find_packages, setup

with open("README.md", encoding="utf-8") as f:
    readme = f.read()

with open("requirements.txt", encoding="utf-8") as f:
    requirements = f.read().split("\n")

with open(os.path.join("cve_bin_tool", "version.py")) as f:
    for line in f:
        if line.startswith("VERSION"):
            VERSION = ast.literal_eval(line.strip().split("=")[-1].strip())
            break

setup_kwargs = dict(
    name="cve-bin-tool",
    version=VERSION,
    description="CVE Binary Checker Tool",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Terri Oda",
    author_email="terri.oda@intel.com",
    maintainer="Terri Oda",
    maintainer_email="terri.oda@intel.com",
    url="https://github.com/intel/cve-bin-tool",
    license="GPLv3",
    keywords=["security", "tools", "CVE"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    install_requires=requirements,
    packages=find_packages(),
    package_data={
        "cve_bin_tool.output_engine": [
            "html_reports/templates/*.html",
            "html_reports/css/*.css",
            "html_reports/js/*.js",
            "print_mode/templates/*.html",
        ]
    },
    entry_points={
        "console_scripts": [
            "cve-bin-tool = cve_bin_tool.cli:main",
            "csv2cve = cve_bin_tool.csv2cve:main",
        ],
        "cve_bin_tool.checker": [
            "{} = cve_bin_tool.checkers.{}:{}".format(
                filename.replace(".py", ""),
                filename.replace(".py", ""),
                "".join(
                    (filename.replace(".py", "") + " checker")
                    .replace("_", " ")
                    .title()
                    .split()
                ),
            )
            for filename in os.listdir(
                os.path.join(
                    os.path.abspath(os.path.dirname(__file__)),
                    "cve_bin_tool",
                    "checkers",
                )
            )
            if filename.endswith(".py") and "__init__" not in filename
        ],
    },
)

setup(**setup_kwargs)



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "bzip2", "version": "1.0.2", "version_strings": ["bzip2-1.0.2"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/bzip2/1.0.4/10.fc7/x86_64/",
        "package_name": "bzip2-1.0.4-10.fc7.x86_64.rpm",
        "product": "bzip2",
        "version": "1.0.4",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "bzip2-1.0.6-13.el7.x86_64.rpm",
        "product": "bzip2",
        "version": "1.0.6",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=keyword-arg-before-vararg
# disabled for python2 compatibility reasons
"""
Extraction of archives
"""
import itertools
import os
import re
import shutil
import sys
import tempfile

from rpmfile.cli import main as rpmextract

from cve_bin_tool.async_utils import (
    ChangeDirContext,
    FileIO,
    aio_glob,
    aio_inpath,
    aio_makedirs,
    aio_mkdtemp,
    aio_rmdir,
    aio_run_command,
    aio_unpack_archive,
    async_wrap,
    run_coroutine,
)

from .error_handler import ErrorHandler, ErrorMode, ExtractionFailed, UnknownArchiveType
from .log import LOGGER

# Run rpmfile in a thread
rpmextract = async_wrap(rpmextract)


class BaseExtractor:
    """Extracts tar, rpm, etc. files"""

    def __init__(self, logger=None, error_mode=ErrorMode.TruncTrace):
        # Sets up logger and if we should extract files or just report
        self.logger = logger or LOGGER.getChild(self.__class__.__name__)
        self.error_mode = error_mode
        self.file_extractors = {
            self.extract_file_tar: {".tgz", ".tar.gz", ".tar", ".tar.xz", ".tar.bz2"},
            self.extract_file_rpm: {".rpm"},
            self.extract_file_deb: {".deb", ".ipk"},
            self.extract_file_cab: {".cab"},
            self.extract_file_apk: {".apk"},
            self.extract_file_zip: {
                ".exe",
                ".zip",
                ".jar",
                ".msi",
                ".egg",
                ".whl",
            },
        }

    def can_extract(self, filename):
        """Check if the filename is something we know how to extract"""
        for extension in itertools.chain(*self.file_extractors.values()):
            if filename.endswith(extension):
                return True
        return False

    @staticmethod
    async def extract_file_tar(filename, extraction_path):
        """Extract tar files"""
        with ErrorHandler(mode=ErrorMode.Ignore) as e:
            await aio_unpack_archive(filename, extraction_path)
        return e.exit_code

    async def extract_file_rpm(self, filename, extraction_path):
        """Extract rpm packages"""
        if sys.platform.startswith("linux"):
            if not await aio_inpath("rpm2cpio") or not await aio_inpath("cpio"):
                await rpmextract("-xC", extraction_path, filename)
            else:
                stdout, stderr, _ = await aio_run_command(["rpm2cpio", filename])
                if stderr or not stdout:
                    return 1
                cpio_path = os.path.join(extraction_path, "data.cpio")
                async with FileIO(cpio_path, "wb") as f:
                    await f.write(stdout)
                stdout, stderr, _ = await aio_run_command(
                    ["cpio", "-idm", "--file", cpio_path]
                )
                if stdout or not stderr:
                    return 1
        else:
            if not await aio_inpath("7z"):
                with ErrorHandler(mode=self.error_mode, logger=self.logger):
                    raise Exception("7z is required to extract rpm files")
            else:
                stdout, stderr, _ = await aio_run_command(["7z", "x", filename])
                if stderr or not stdout:
                    return 1
                filenames = await aio_glob(os.path.join(extraction_path, "*.cpio"))
                filename = filenames[0]

                stdout, stderr, _ = await aio_run_command(["7z", "x", filename])
                if stderr or not stdout:
                    return 1
        return 0

    async def extract_file_deb(self, filename, extraction_path):
        """Extract debian packages"""
        if not await aio_inpath("ar"):
            with ErrorHandler(mode=self.error_mode, logger=self.logger):
                raise Exception("'ar' is required to extract deb files")
        else:
            stdout, stderr, _ = await aio_run_command(["ar", "x", filename])
            if stderr:
                return 1
            datafile = await aio_glob(os.path.join(extraction_path, "data.tar.*"))
            with ErrorHandler(mode=ErrorMode.Ignore) as e:
                await aio_unpack_archive(datafile[0], extraction_path)
            return e.exit_code

    async def extract_file_apk(self, filename, extraction_path):
        """Check whether it is alpine or android package"""

        is_tar = True
        process_can_fail = True
        if await aio_inpath("unzip"):
            stdout, stderr, return_code = await aio_run_command(
                ["unzip", "-l", filename], process_can_fail
            )
            if return_code == 0:
                is_tar = False
        elif await aio_inpath("7z"):
            stdout, stderr, return_code = await aio_run_command(
                ["7z", "t", filename], process_can_fail
            )
            if re.search(b"Type = Zip", stdout):
                is_tar = False
        elif await aio_inpath("zipinfo"):
            stdout, stderr, return_code = await aio_run_command(
                ["zipinfo", filename], process_can_fail
            )
            if return_code == 0:
                is_tar = False
        elif await aio_inpath("file"):
            stdout, stderr, return_code = await aio_run_command(
                ["file", filename], process_can_fail
            )
            if re.search(b"Zip archive data", stdout):
                is_tar = False
        if is_tar:
            self.logger.debug(f"Extracting {filename} as a tar.gzip file")
            with ErrorHandler(mode=ErrorMode.Ignore) as e:
                await aio_unpack_archive(filename, extraction_path, format="gztar")
            return e.exit_code
        else:
            return await self.extract_file_zip(filename, extraction_path)

    @staticmethod
    async def extract_file_cab(filename, extraction_path):
        """Extract cab files"""
        if sys.platform.startswith("linux"):
            if not await aio_inpath("cabextract"):
                raise Exception("'cabextract' is required to extract cab files")
            else:
                stdout, stderr, _ = await aio_run_command(
                    ["cabextract", "-d", extraction_path, filename]
                )
                if stderr or not stdout:
                    return 1
        else:
            stdout, stderr, _ = await aio_run_command(
                ["Expand", filename, "-R -F:*", extraction_path]
            )
            if stderr or not stdout:
                return 1
        return 0

    @staticmethod
    async def extract_file_zip(filename, extraction_path, process_can_fail=True):
        """Extract zip files"""

        is_exe = filename.endswith(".exe")
        if await aio_inpath("unzip"):
            stdout, stderr, _ = await aio_run_command(
                ["unzip", "-n", "-d", extraction_path, filename], process_can_fail
            )
            if stderr or not stdout:
                if is_exe:
                    return 0  # not all .exe files are zipfiles, no need for error
                return 1
        elif await aio_inpath("7z"):
            stdout, stderr, _ = await aio_run_command(
                ["7z", "x", filename], process_can_fail
            )
            if stderr or not stdout:
                if is_exe:
                    return 0  # not all .exe files are zipfiles, no need for error
                return 1
        else:
            with ErrorHandler(mode=ErrorMode.Ignore) as e:
                await aio_unpack_archive(filename, extraction_path)
            return e.exit_code
        return 0


class TempDirExtractorContext(BaseExtractor):
    """Extracts tar, rpm, etc. files"""

    def __init__(self, raise_failure=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.tempdir = None
        self.raise_failure = raise_failure

    async def aio_extract(self, filename):
        """Run the extractor"""
        # Resolve path in case of cwd change
        filename = os.path.abspath(filename)
        for extractor in self.file_extractors:
            for extension in self.file_extractors[extractor]:
                if filename.endswith(extension):
                    extracted_path = os.path.join(
                        self.tempdir, f"{os.path.basename(filename)}.extracted"
                    )
                    if os.path.exists(extracted_path):
                        await aio_rmdir(extracted_path)
                    await aio_makedirs(extracted_path, 0o700)
                    async with ChangeDirContext(extracted_path):
                        if await extractor(filename, extracted_path) != 0:
                            if self.raise_failure:
                                with ErrorHandler(
                                    mode=self.error_mode, logger=self.logger
                                ):
                                    raise ExtractionFailed(filename)
                            else:
                                self.logger.warning(f"Failure extracting {filename}")
                        else:
                            self.logger.debug(
                                f"Extracted {filename} to {extracted_path}"
                            )
                    return extracted_path
        with ErrorHandler(mode=self.error_mode, logger=self.logger):
            raise UnknownArchiveType(filename)

    async def __aenter__(self):
        """Create a temporary directory to extract files to."""
        self.tempdir = await aio_mkdtemp(prefix="cve-bin-tool-")
        return self

    async def __aexit__(self, exc_type, exc, exc_tb):
        """Removes all extraction directories that need to be cleaned up."""
        # removing directory can raise exception so wrap it around ErrorHandler.
        with ErrorHandler(mode=self.error_mode, logger=self.logger):
            await aio_rmdir(self.tempdir)

    def extract(self, filename):
        return run_coroutine(self.aio_extract(filename))

    def __enter__(self):
        self.tempdir = tempfile.mkdtemp(prefix="cve-bin-tool-")
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        with ErrorHandler(mode=self.error_mode, logger=self.logger):
            shutil.rmtree(self.tempdir)


def Extractor(*args, **kwargs):
    """Provides a context which extraction is done in"""
    return TempDirExtractorContext(*args, **kwargs)



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE checker for WebKitGTK

https://www.cvedetails.com/product/20596/Webkitgtk-Webkitgtk.html?vendor_id=11350
"""

from . import Checker


class WebkitgtkChecker(Checker):
    CONTAINS_PATTERNS = [r"www.webkitgtk.org"]
    FILENAME_PATTERNS = [
        r"libwebkitgtk",
        r"WebKitGTK-([0-9]+\.[0-9]+).mo",
    ]
    VERSION_PATTERNS = [
        r"webkitgtk-([0-9]+\.[0-9]+\.[0-9]+)",
    ]
    VENDOR_PRODUCT = [("webkitgtk", "webkitgtk")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for bubblewrap

https://www.cvedetails.com/vendor/16078/?q=Bubblewrap+Project

"""
from cve_bin_tool.checkers import Checker


class BubblewrapChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"bwrap"]
    VERSION_PATTERNS = [r"bubblewrap ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [
        ("bubblewrap_project", "bubblewrap"),
        ("projectatomic", "bubblewrap"),
    ]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for wireshark

https://www.cvedetails.com/product/8292/Wireshark-Wireshark.html?vendor_id=4861

"""
from cve_bin_tool.checkers import Checker


class WiresharkChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"rawshark"]
    VERSION_PATTERNS = [r"Wireshark ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("wireshark", "wireshark")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for openjpeg

https://www.cvedetails.com/product/22664/Openjpeg-Openjpeg.html?vendor_id=12064

"""
from . import Checker


class OpenjpegChecker(Checker):
    CONTAINS_PATTERNS = [
        r"OpenJPEG cannot encode raw components with bit depth higher than 16 bits."
    ]
    FILENAME_PATTERNS = [
        r"extract_j2k_from_mj2",
        r"frames_to_mj2",
        r"image_to_j2k",
        r"j2k_dump",
        r"j2k_to_image",
        r"mj2_to_frames",
        r"wrap_j2k_in_mj2",
    ]
    VERSION_PATTERNS = [
        r"openjpeg-([0-9]+\.[0-9]+\.[0-9]+)",
    ]
    VENDOR_PRODUCT = [("openjpeg", "openjpeg")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE checker for pspp

https://www.cvedetails.com/product/38732/GNU-Pspp.html?vendor_id=72

"""
from cve_bin_tool.checkers import Checker


class PsppChecker(Checker):

    CONTAINS_PATTERNS = [
        r"The PSPP language identifier for the data associated with this window \(e\.g\. dataset name\)",
        r"bug-gnu-pspp@gnu\.org",
    ]
    FILENAME_PATTERNS = [r"libpspp"]
    VERSION_PATTERNS = [r"libpspp-([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("gnu", "pspp")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE-bin-tool Strings tests
"""
import os
from test.utils import event_loop

import pytest

from cve_bin_tool.async_utils import aio_run_command
from cve_bin_tool.strings import Strings

ASSETS_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), "assets")


class TestStrings:
    """Tests the CVE Bin Tool Strings"""

    @classmethod
    def setup_class(cls):
        cls.strings = Strings()

    @pytest.mark.asyncio
    async def _parse_test(self, filename):
        """Helper function to parse a binary file and check whether
        the given string is in the parsed result"""
        self.strings.filename = os.path.join(ASSETS_PATH, filename)
        binutils_strings, _, _ = await aio_run_command(
            ["strings", self.strings.filename]
        )
        ours = await self.strings.aio_parse()
        for theirs in binutils_strings.splitlines():
            assert theirs.decode("utf-8") in ours

    @pytest.mark.asyncio
    async def test_curl_7_34_0(self):
        """Stringsing test-curl-7.34.0.out"""
        await self._parse_test("test-curl-7.34.0.out")

    @pytest.mark.asyncio
    async def test_kerberos_1_15_1(self):
        """Stringsing test-kerberos-5-1.15.1.out"""
        await self._parse_test("test-kerberos-5-1.15.1.out")



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for qt

https://www.cvedetails.com/product/10758/QT-QT.html?vendor_id=6363
https://www.cvedetails.com/product/24410/Digia-QT.html?vendor_id=12593

"""
from cve_bin_tool.checkers import Checker


class QtChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [
        r"libqt-mt.so",
        r"libQtTest.so",
    ]
    VERSION_PATTERNS = [
        r"Qt ([0-9]+\.[0-9]+\.[0-9]+)",
        r"QTest library ([0-9]+\.[0-9]+\.[0-9]+)",
    ]
    VENDOR_PRODUCT = [("qt", "qt"), ("digia", "qt")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for busybox

https://www.cvedetails.com/product/7452/Busybox-Busybox.html?vendor_id=4282

"""
from cve_bin_tool.checkers import Checker


class BusyboxChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"busybox"]
    VERSION_PATTERNS = [r"BusyBox v([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("busybox", "busybox")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "bubblewrap",
        "version": "0.4.1",
        "version_strings": ["bubblewrap 0.4.1"],
    },
    {
        "product": "bubblewrap",
        "version": "0.4.0",
        "version_strings": ["bubblewrap 0.4.0"],
    },
]
package_test_data = [
    {
        "url": "http://archive.ubuntu.com/ubuntu/pool/main/b/bubblewrap/",
        "package_name": "bubblewrap_0.4.1-3_amd64.deb",
        "product": "bubblewrap",
        "version": "0.4.1",
    },
    {
        "url": "http://mirror.centos.org/centos/8/BaseOS/aarch64/os/Packages/",
        "package_name": "bubblewrap-0.4.0-1.el8.aarch64.rpm",
        "product": "bubblewrap",
        "version": "0.4.0",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "libxslt",
        "version": "1.1.28",
        "version_strings": ["libxslt.so.1.1.28", "xsltproc.debug"],
    }
]
package_test_data = [
    {
        "url": "http://mirror.centos.org/altarch/7/os/aarch64/Packages/",
        "package_name": "libxslt-1.1.28-6.el7.aarch64.rpm",
        "product": "libxslt",
        "version": "1.1.28",
    },
    {
        "url": "http://xmlsoft.org/sources/",
        "package_name": "libxslt-1.1.33-0rc1.fc26.x86_64.rpm",
        "product": "libxslt",
        "version": "1.1.33",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "cryptsetup",
        "version": "2.3.6",
        "version_strings": [
            "Legacy offline reencryption already in-progress. Use cryptsetup-reencrypt utility.",
            "Only LUKS2 format is currently supported. Please use cryptsetup-reencrypt tool for LUKS1.",
            "cryptsetup 2.3.6",
        ],
    },
    {
        "product": "cryptsetup",
        "version": "2.0.3",
        "version_strings": [
            "Legacy offline reencryption already in-progress. Use cryptsetup-reencrypt utility.",
            "Only LUKS2 format is currently supported. Please use cryptsetup-reencrypt tool for LUKS1.",
            "cryptsetup 2.0.3",
        ],
    },
]
package_test_data = [
    {
        "url": "https://download-ib01.fedoraproject.org/pub/fedora/linux/development/rawhide/Everything/aarch64/os/Packages/c/",
        "package_name": "cryptsetup-2.3.6-1.fc35.aarch64.rpm",
        "product": "cryptsetup",
        "version": "2.3.6",
    },
    {
        "url": " 	http://mirror.centos.org/altarch/7/os/aarch64/Packages/",
        "package_name": "cryptsetup-2.0.3-6.el7.aarch64.rpm",
        "product": "cryptsetup",
        "version": "2.0.3",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import os
from collections import Counter, defaultdict
from datetime import datetime
from typing import Dict, List, Union

import plotly.graph_objects as go
from jinja2 import Environment, FileSystemLoader, select_autoescape
from jinja2.environment import Template

from cve_bin_tool.merge import MergeReports

from ..log import LOGGER
from ..util import CVEData, ProductInfo, Remarks
from ..version import VERSION
from .print_mode import html_print_mode
from .util import group_cve_by_remark

SEVERITY_TYPES_COLOR = {
    "CRITICAL": "red",
    "HIGH": "blue",
    "MEDIUM": "yellow",
    "LOW": "green",
    "UNKNOWN": "black",
}


def render_cves(
    hid: str, cve_row: Template, tag: str, cves: List[Dict[str, str]]
) -> str:
    """Return rendered form of CVEs using cve_row template

    Args:
        hid (str): Unique id for each product
        cve_row (Template): JinjaTemplate to be used for rendering
        tag (str): Marked by user. (default: New) Can be anything [NEW, MITIGATED, IGNORED, UNEXPLORED]
        cves (List[Dict[str, str]]): List of CVEs present in the products.

    Returns:
        str: CVE(s) in rendered form
    """
    list_cves = []
    for i, cve in enumerate(cves):
        # render CVE template with data and add to list_cves
        list_cves.append(
            cve_row.render(
                cve_number=cve["cve_number"],
                severity=cve["severity"],
                description=cve["description"],
                tag=tag,
                var_id=f"{hid}{i}{tag}",
                fix_id=hid,
            )
        )
    return "".join(list_cves)


def output_html(
    all_cve_data: Dict[ProductInfo, CVEData],
    scanned_dir: str,
    filename: str,
    theme_dir: str,
    total_files: int,
    products_with_cve: int,
    products_without_cve: int,
    merge_report: Union[None, MergeReports],
    logger: LOGGER,
    outfile,
):
    """Returns a HTML report for CVE's"""

    # Step 1: Load all the templates

    # Root folder where html_reports is present
    root = os.path.dirname(os.path.abspath(__file__))

    # Template Directory contains all the html files
    templates_dir = os.path.join(root, "html_reports")
    templates_env = Environment(
        loader=FileSystemLoader(templates_dir),
        autoescape=select_autoescape(
            enabled_extensions=("html"), disabled_extensions=("css,js")
        ),
    )

    temp_base = "templates/base.html"
    temp_dash = "templates/dashboard.html"
    temp_product = "templates/row_product.html"
    temp_cve = "templates/row_cve.html"
    temp_intermediate = "templates/intermediate.html"

    base = templates_env.get_template(temp_base)
    dashboard = templates_env.get_template(temp_dash)
    cve_row = templates_env.get_template(temp_cve)
    product_row = templates_env.get_template(temp_product)
    # Load merge template if the report is generated from intermediate reports
    if merge_report:
        merged = templates_env.get_template(temp_intermediate)
        (
            products_trace,
            total_files_trace,
            intermediate_timeline,
            severity_trace,
        ) = load_timeline_from_merged(merge_report)

        # Intermediate Graphs Rendering
        intermediate = merged.render(
            products_trace=products_trace.to_html(
                full_html=False, include_plotlyjs=False
            ),
            total_files_trace=total_files_trace.to_html(
                full_html=False, include_plotlyjs=False
            ),
            intermediate_timeline=intermediate_timeline.to_html(
                full_html=False, include_plotlyjs=False
            ),
            severity_trace=severity_trace.to_html(
                full_html=False, include_plotlyjs=False
            ),
        )
    else:
        intermediate = None

    # Step 2: Prepare Charts
    # Start generating graph with the data

    # dash graph1: Products Vulnerability Graph
    product_pie = go.Figure(
        data=[
            go.Pie(
                labels=["Vulnerable", "No Known Vulnerability"],
                values=[products_with_cve, products_without_cve],
                hole=0.4,
            )
        ]
    )

    # Chart configuration for product_pie
    product_pie.update_layout(
        autosize=True,
        legend_orientation="h",
    )
    product_pie.update_traces(
        hoverinfo="label+percent",
        textinfo="value",
        textfont_size=14,
        marker=dict(
            colors=["#d80032", "#1a936f"],
            line=dict(color="white", width=2),
        ),
    )

    # dash graph2: Product CVE's Graph
    cve_bar = go.Figure()
    for product_info, cve_data in all_cve_data.items():
        # Check if product contains CVEs
        if cve_data["cves"]:
            if product_info.vendor != "UNKNOWN":
                cve_bar.add_trace(
                    go.Bar(
                        x=[
                            f"{product_info.vendor}-{product_info.product}({product_info.version})"
                        ],
                        y=[
                            0
                            if cve_data["cves"][0][1] == "UNKNOWN"
                            else len(cve_data["cves"])
                        ],
                        name=f"{product_info.product}-{product_info.version}",
                    )
                )
            else:
                cve_bar.add_trace(
                    go.Bar(
                        x=[f"{product_info.product}({product_info.version})"],
                        y=[
                            0
                            if cve_data["cves"][0][1] == "UNKNOWN"
                            else len(cve_data["cves"])
                        ],
                        name=f"{product_info.product}-{product_info.version}",
                    )
                )

    # Chart configuration for cve_bar
    cve_bar.update_layout(
        yaxis_title="Number of CVE's",
    )

    all_paths = defaultdict(list)

    star_warn = ""
    products_found = []
    # List of Products
    for product_info, cve_data in all_cve_data.items():
        # Check if product contains CVEs
        if cve_data["cves"]:

            # group product wise cves on the basis of remarks
            cve_by_remark = group_cve_by_remark(cve_data["cves"])

            # hid is unique for each product
            if product_info.vendor != "UNKNOWN":
                hid = f"{product_info.vendor}{product_info.product}{''.join(product_info.version.split('.'))}"
            else:
                hid = (
                    f"{product_info.product}{''.join(product_info.version.split('.'))}"
                )
            new_cves = render_cves(
                hid,
                cve_row,
                "NEW",
                cve_by_remark[Remarks.NewFound],
            )
            mitigated_cves = render_cves(
                hid,
                cve_row,
                "MITIGATED",
                cve_by_remark[Remarks.Mitigated],
            )
            confirmed_cves = render_cves(
                hid,
                cve_row,
                "CONFIRMED",
                cve_by_remark[Remarks.Confirmed],
            )
            unexplored_cves = render_cves(
                hid,
                cve_row,
                "UNEXPLORED",
                cve_by_remark[Remarks.Unexplored],
            )
            ignored_cves = render_cves(
                hid,
                cve_row,
                "IGNORED",
                cve_by_remark[Remarks.Ignored],
            )

            analysis_data = Counter(cve.severity for cve in cve_data["cves"])

            # initialize a figure object for Analysis Chart
            analysis_pie = go.Figure(
                data=[
                    go.Pie(
                        labels=list(analysis_data.keys()),
                        values=list(analysis_data.values()),
                        hole=0.4,
                    )
                ]
            )
            colors_avail = {
                "CRITICAL": "#f25f5c",
                "HIGH": "#ee6c4d",
                "MEDIUM": "#f4d35e",
                "LOW": "#90a955",
                "UNKNOWN": "#808080",
            }
            colors = [colors_avail[label] for label in analysis_data.keys()]
            analysis_pie.update_traces(
                hoverinfo="label+percent",
                textinfo="value",
                textfont_size=14,
                marker=dict(
                    colors=colors,
                    line=dict(color="white", width=2),
                ),
            )
            analysis_pie.update_layout(
                autosize=True,
                height=300,
                legend_orientation="h",
                margin=dict(l=0, r=20, t=0, b=0),
                # paper_bgcolor="LightSteelBlue",
            )

            products_found.append(
                product_row.render(
                    vendor=product_info.vendor,
                    name=product_info.product,
                    version=product_info.version,
                    cve_count=0
                    if cve_data["cves"][0][1] == "UNKNOWN"
                    else len(cve_data["cves"]),
                    severity_analysis=analysis_pie.to_html(
                        full_html=False, include_plotlyjs=False
                    ),
                    fix_id=hid,
                    paths=cve_data["paths"],
                    len_paths=len(cve_data["paths"]),
                    new_cves=new_cves,
                    mitigated_cves=mitigated_cves,
                    confirmed_cves=confirmed_cves,
                    unexplored_cves=unexplored_cves,
                    ignored_cves=ignored_cves,
                )
            )

            if "*" in product_info.vendor:
                star_warn = "* vendors guessed by the tool"

            # update all_paths
            for path in cve_data["paths"]:
                all_paths[path].append(hid)

    # Dashboard Rendering
    dashboard = dashboard.render(
        graph_cves=cve_bar.to_html(full_html=False, include_plotlyjs=False),
        graph_products=product_pie.to_html(full_html=False, include_plotlyjs=False),
        directory=scanned_dir,
        total_files=total_files,
        products_with_cve=products_with_cve,
        products_without_cve=products_without_cve,
    )

    # try to load the bigger files just before the generation of report

    # css template names
    css_main = "css/main.css"
    css_bootstrap = "css/bootstrap.css"

    style_main = templates_env.get_template(css_main)
    style_bootstrap = templates_env.get_template(css_bootstrap)

    # js template names
    js_main = "js/main.js"
    js_bootstrap = "js/bootstrap.js"
    js_plotly = "js/plotly.js"
    js_jquery = "js/jquery.js"

    script_main = templates_env.get_template(js_main)
    script_bootstrap = templates_env.get_template(js_bootstrap)
    script_plotly = templates_env.get_template(js_plotly)
    script_jquery = templates_env.get_template(js_jquery)

    # Render the base html to generate report
    outfile.write(
        base.render(
            date=datetime.now().strftime("%d %b %Y"),
            dashboard=dashboard,
            intermediate=intermediate,
            scanned_dir=scanned_dir,
            all_paths=all_paths,
            print_mode=html_print_mode(
                all_cve_data,
                scanned_dir,
                products_with_cve,
                products_without_cve,
                total_files,
                star_warn,
                merge_report,
                version=VERSION,
                full_html=False,
            ),
            products_found="".join(products_found),
            version=VERSION,
            star_warn=star_warn,
            style_main=style_main.render(),
            style_bootstrap=style_bootstrap.render(),
            script_main=script_main.render(),
            script_jquery=script_jquery.render(),
            script_bootstrap=script_bootstrap.render(),
            script_plotly=script_plotly.render(),
        )
    )


def get_intermediate_label(metadata: Dict[str, str]) -> str:
    """Return the x-axis timestamp label for intermediate reports"""

    if metadata["tag"]:
        timestamp_label = f'{datetime.strptime(metadata["timestamp"], "%Y-%m-%d.%H-%M-%S").strftime("%d %b %H:%M")}-{metadata["tag"]}'
    else:
        timestamp_label = f'{datetime.strptime(metadata["timestamp"], "%Y-%m-%d.%H-%M-%S").strftime("%d %b %H:%M")}'

    return timestamp_label


def load_timeline_from_merged(merge_report: MergeReports):
    """Load metadata from intermediate reports and return graphs to construct the HTML file"""

    timestamps = []
    products_with_cve = []
    products_without_cve = []
    total_files = []
    severity_count_list = []

    for inter_file in merge_report.intermediate_cve_data:
        products_with_cve.append(inter_file["metadata"]["products_with_cve"])
        products_without_cve.append(inter_file["metadata"]["products_without_cve"])
        total_files.append(inter_file["metadata"]["total_files"])
        severity_count_list.append(inter_file["metadata"]["severity"])
        timestamps.append(get_intermediate_label(inter_file["metadata"]))

    intermediate_cve_scanner_list = merge_report.get_intermediate_cve_scanner(
        merge_report.intermediate_cve_data, merge_report.score
    )

    cve_bar_data = []
    cve_product_list = set()

    # Create a intermediate data wise product dictionary
    for cve_scanner in intermediate_cve_scanner_list:

        data = dict()
        for product_info, cve_data in cve_scanner.all_cve_data.items():
            # Check if product contains CVEs
            if cve_data["cves"]:
                x = f"{product_info.product}({product_info.version})"
                y = 0 if cve_data["cves"][0][1] == "UNKNOWN" else len(cve_data["cves"])
                data[x] = y
                cve_product_list.add(x)

        cve_bar_data.append(data)

    intermediate_timeline = go.Figure()
    cve_product_list = list(cve_product_list)
    # Add product wise stacked bar
    for key in cve_product_list:
        intermediate_timeline.add_trace(
            go.Bar(
                name=key,
                x=timestamps,
                y=[data[key] if key in data else 0 for data in cve_bar_data],
            )
        )

    # Chart configuration for intermediate_timeline
    intermediate_timeline.update_layout(
        autosize=True,
        legend_orientation="v",
        barmode="stack",
        yaxis_title="Products with CVE",
        height=400,
    )

    # Graph for severity count
    severity_trace = go.Figure()
    for severity_type in list(SEVERITY_TYPES_COLOR.keys()):
        severity_trace.add_trace(
            go.Scatter(
                name=severity_type,
                x=timestamps,
                y=[count[severity_type] for count in severity_count_list],
                line=dict(color=SEVERITY_TYPES_COLOR[severity_type]),
            )
        )

    # Chart configuration for severity traces
    severity_trace.update_layout(
        autosize=True,
        legend_orientation="v",
        yaxis_title="Severity",
        height=300,
    )

    # Graph for product with/without cves count
    products_trace = go.Figure()
    products_trace.add_trace(
        go.Scatter(
            name="Products with CVE",
            x=timestamps,
            y=products_with_cve,
            xperiodalignment="middle",
        )
    )
    products_trace.add_trace(
        go.Scatter(
            name="Products without CVE",
            x=timestamps,
            y=products_without_cve,
            xperiodalignment="middle",
        ),
    )
    products_trace.update_layout(
        autosize=True,
        legend_orientation="v",
        yaxis_title="Products",
        height=300,
    )

    # Graph for total_files in each intermediate file
    total_files_trace = go.Figure()
    total_files_trace.add_trace(
        go.Scatter(
            name="Total Files",
            x=timestamps,
            y=total_files,
            xperiodalignment="middle",
        )
    )
    total_files_trace.update_layout(
        autosize=True,
        legend_orientation="v",
        yaxis_title="Total Files",
        height=300,
    )

    return products_trace, total_files_trace, intermediate_timeline, severity_trace



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "haproxy",
        "version": "1.7.3",
        "version_strings": ["HA-Proxy version 1.7.3"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/haproxy/1.8.4/2.fc28/x86_64/",
        "package_name": "haproxy-1.8.4-2.fc28.x86_64.rpm",
        "product": "haproxy",
        "version": "1.8.4",
    },
    {
        "url": "https://mirrors.edge.kernel.org/centos/7/os/x86_64/Packages/",
        "package_name": "haproxy-1.5.18-9.el7.x86_64.rpm",
        "product": "haproxy",
        "version": "1.5.18",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "polarssl",
        "version": "1.3.6",
        "version_strings": ["libpolarssl.so.1.3.6.debug"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/polarssl/1.3.7/2.fc21/x86_64/",
        "package_name": "polarssl-1.3.7-2.fc21.x86_64.rpm",
        "product": "polarssl",
        "version": "1.3.7",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

"""
CVE checker for Python
References:
https://www.cvedetails.com/vulnerability-list/vendor_id-10210/product_id-18230/Python-Python.html


"""
import re

from cve_bin_tool.checkers import Checker
from cve_bin_tool.util import regex_find


class PythonChecker(Checker):
    CONTAINS_PATTERNS = [
        r"Fatal Python error: unable to decode the command line argument",
        r"Internal error in the Python interpreter",
        r"CPython",
    ]
    FILENAME_PATTERNS = [r"python"]
    VERSION_PATTERNS = [r"python([23]+\.[0-9])"]
    VENDOR_PRODUCT = [("python_software_foundation", "python"), ("python", "python")]

    def get_version(self, lines, filename):
        # we will try to find python3+ as well as python2+

        # currently regex will probably find a single string "lib/python3.6"
        # where 3.6 is the version similarly "lib/python2.7" where 2.7 is the version
        version_info = super().get_version(lines, filename)

        # we will check if the guess returned some version probably 3.6 or 2.7 in our example
        # return version_info
        if "version" in version_info and version_info["version"] != "UNKNOWN":

            # we will update our regex to something more precise 3.6.d
            # where d is unknown and we will find d. which will return 3.6.9 or some other version
            version_pattern = [
                rf"([{version_info['version'][0]}]+\.[{version_info['version'][2]}]+\.[0-9]+)"
            ]
            version_regex = list(map(re.compile, version_pattern))
            new_version = regex_find(lines, version_regex)

            # we will return this result
            version_info["version"] = new_version

        # else guess was unknown so we update our regex
        elif version_info:
            version_pattern = [
                r"version: ([23]+\.[0-9]+\.[0-9])+",
                r"Python ([23]+\.[0-9]+\.[0-9])+",
            ]
            version_regex = list(map(re.compile, version_pattern))
            new_version = regex_find(lines, version_regex)

            version_info["version"] = new_version
        return version_info



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "subversion",
        "version": "1.13.0",
        "version_strings": [
            r"subversion-1.13.0",
            r"Working copy locked; if no other Subversion client is currently using the working copy, try running 'svn cleanup' without the --remove",
            r"Working copy locked; try running 'svn cleanup' on the root of the working copy ('%s') instead.",
        ],
    }
]
package_test_data = [
    {
        "url": "http://ports.ubuntu.com/pool/main/s/subversion/",
        "package_name": "subversion_1.9.3-2ubuntu1_arm64.deb",
        "product": "subversion",
        "version": "1.9.3",
    }
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for tcpdump

https://www.cvedetails.com/product/10494/Tcpdump-Tcpdump.html?vendor_id=6197

"""
from cve_bin_tool.checkers import Checker


class TcpdumpChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"tcpdump"]
    VERSION_PATTERNS = [r"tcpdump-([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("tcpdump", "tcpdump")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "bind", "version": "9.10.2", "version_strings": ["BIND 9.10.2"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/bind/9.10.3/1.fc23/x86_64/",
        "package_name": "bind-9.10.3-1.fc23.x86_64.rpm",
        "product": "bind",
        "version": "9.10.3",
    },
    {
        "url": "http://mirror.centos.org/centos/7/os/x86_64/Packages/",
        "package_name": "bind-9.11.4-26.P2.el7.x86_64.rpm",
        "product": "bind",
        "version": "9.11.4",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for radare2

https://www.cvedetails.com/product/36037/Radare-Radare2.html?vendor_id=16137

"""
from cve_bin_tool.checkers import Checker


class Radare2Checker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"rafind2"]
    VERSION_PATTERNS = [r"rafind2 v([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("radare", "radare2")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "ntp",
        "version": "4.2.8p15",
        "version_strings": ["ntpd 4.2.8p15", "ntpdc 4.2.8p15", "ntpq 4.2.8p15"],
    },
    {
        "product": "ntp",
        "version": "4.2.6p5",
        "version_strings": ["ntpd 4.2.6p5", "ntpdc 4.2.6p5", "ntpq 4.2.6p5"],
    },
]
package_test_data = [
    {
        "url": "https://download-ib01.fedoraproject.org/pub/fedora/linux/releases/33/Everything/x86_64/os/Packages/n/",
        "package_name": "ntp-4.2.8p15-3.fc33.x86_64.rpm",
        "product": "ntp",
        "version": "4.2.8p15",
    },
    {
        "url": "https://rpmfind.net/linux/mageia/distrib/5/x86_64/media/core/release/",
        "package_name": "ntp-4.2.6p5-24.mga5.x86_64.rpm",
        "product": "ntp",
        "version": "4.2.6p5",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for nessus

References:
https://www.cvedetails.com/vulnerability-list/vendor_id-12865/product_id-27428/Tenable-Nessus.html
"""
from cve_bin_tool.checkers import Checker


class NessusChecker(Checker):
    CONTAINS_PATTERNS = [
        r"you have deleted older versions nessus libraries from your system",
        r"server_info_nessusd_version",
        r"nessuslib_version",
        r"nessus_lib_version",
    ]
    FILENAME_PATTERNS = [r"libnessus"]
    VERSION_PATTERNS = [
        r"Nessus ([0-9]+\.[0-9]+\.[0-9]+)",
        r"libnessus.so.([0-9]+\.[0-9]+\.[0-9]+)",  # patterns like this aren't ideal
    ]
    VENDOR_PRODUCT = [("tenable", "nessus")]


"""
Using filenames (containing patterns like '.so' etc.) in the binaries as VERSION_PATTERNS aren't ideal.
The reason behind this is that these might depend on who packages the file (like it 
might work on fedora but not on ubuntu)
"""



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import sqlite3
from os.path import join

from rich.progress import track

from cve_bin_tool.cvedb import DBNAME, DISK_LOCATION_DEFAULT


class VendorFetch:
    """
    This class is for reading vendor from the database for packages that doesn't have vendor info
    """

    def __init__(self) -> None:
        self.dbname: str = join(DISK_LOCATION_DEFAULT, DBNAME)

    def get_vendor_product_pairs(self, package_names):
        vendor_package_pairs = []
        query = """
        SELECT DISTINCT vendor FROM cve_range
        WHERE product=? 
        """
        # For python package checkers we don't need the progress bar running
        if type(package_names) != list:
            package_name = package_names  # Since package names will only have the 'package name' in it
            self.cursor.execute(query, [package_name])
            vendors = list(map(lambda x: x[0], self.cursor.fetchall()))
            for vendor in vendors:
                if vendor != "":
                    vendor_package_pairs.append(
                        {
                            "vendor": vendor,
                            "product": package_name,
                        }
                    )
        else:
            for package_name in track(
                package_names, description="Processing the given list...."
            ):
                self.cursor.execute(query, [package_name["name"].lower()])
                vendors = list(map(lambda x: x[0], self.cursor.fetchall()))
                for vendor in vendors:
                    if vendor != "":
                        vendor_package_pairs.append(
                            {
                                "vendor": vendor,
                                "product": package_name["name"],
                            }
                        )
        return vendor_package_pairs

    def __enter__(self):
        self.connection = sqlite3.connect(self.dbname)
        self.connection.row_factory = sqlite3.Row
        self.cursor = self.connection.cursor()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cursor.close()
        self.connection.close()



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "libssh2",
        "version": "1.8.0",
        "version_strings": ["SSH-2.0-libssh2_1.8.0"],
    },
    {
        "product": "libssh2",
        "version": "1.9.0",
        "version_strings": ["SSH-2.0-libssh2_1.9.0"],
    },
]
package_test_data = [
    {
        "url": "https://download-ib01.fedoraproject.org/pub/epel/8/Everything/aarch64/Packages/l/",
        "package_name": "libssh2-1.9.0-5.el8.aarch64.rpm",
        "product": "libssh2",
        "version": "1.9.0",
    },
    {
        "url": "http://mirror.centos.org/altarch/7/os/aarch64/Packages/",
        "package_name": "libssh2-1.8.0-4.el7.aarch64.rpm",
        "product": "libssh2",
        "version": "1.8.0",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for postgresql

https://www.cvedetails.com/product/575/Postgresql-Postgresql.html?vendor_id=336

"""
from cve_bin_tool.checkers import Checker


class PostgresqlChecker(Checker):
    # FIXME: fix contains pattern
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [r"psql"]
    VERSION_PATTERNS = [r"PostgreSQL ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("postgresql", "postgresql")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for openldap

https://www.cvedetails.com/product/755/Openldap-Openldap.html?vendor_id=439

"""
from cve_bin_tool.checkers import Checker


class OpenldapChecker(Checker):
    FILENAME_PATTERNS = [r"ldapsearch"]
    VERSION_PATTERNS = [r"ldapsearch ([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("openldap", "openldap")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for samba

https://www.cvedetails.com/product/171/Samba-Samba.html?vendor_id=102

https://www.samba.org/~ab/output/htmldocs/Samba3-HOWTO/compiling.html also
lists "winbindd" and "nmbd" as necessary to configure in startup along with smbd.
But these files do not have the strings which match the signatures in regex. 
That is why they have not been added in FILENAME_PATTERNS. 

"""
from cve_bin_tool.checkers import Checker


class SambaChecker(Checker):
    CONTAINS_PATTERNS = []
    FILENAME_PATTERNS = [
        r"smbd",
        r"smbtree",
        r"smbget",
        r"smbstatus",
        r"smbspool",
        r"smbpasswd",
        r"smbcquotas",
        r"smbcontrol",
        r"smbclient",
        r"smbcacls",
        r"sharesec",
    ]
    VERSION_PATTERNS = [r"SAMBA_([0-9]+\.[0-9]+\.[0-9]+)"]
    VENDOR_PRODUCT = [("samba", "samba")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

import os
from collections import ChainMap
from logging import Logger
from typing import Any, Mapping

import toml
import yaml

from cve_bin_tool.error_handler import ErrorHandler, ErrorMode, UnknownConfigType
from cve_bin_tool.log import LOGGER


class ConfigParser:
    # Key-value pair of config data ex: {"extract": True, "directory": "test/assets"}
    config_data: Mapping[str, Any]

    def __init__(
        self, filename: str, logger: Logger = None, error_mode=ErrorMode.TruncTrace
    ):
        self.filename = os.path.abspath(filename)
        self.logger = logger or LOGGER.getChild(self.__class__.__name__)
        self.error_mode = error_mode
        self.config_data = {}

    def parse_config(self) -> Mapping[str, Any]:
        if not os.path.isfile(self.filename):
            with ErrorHandler(mode=self.error_mode):
                raise FileNotFoundError(self.filename)
        if self.filename.endswith(".toml"):
            with open(self.filename) as f:
                raw_config_data = toml.load(f)
                self.config_data = ChainMap(*raw_config_data.values())
        elif self.filename.endswith(".yaml"):
            with open(self.filename) as f:
                raw_config_data = yaml.safe_load(f)
                self.config_data = ChainMap(*raw_config_data.values())
        else:
            with ErrorHandler(mode=self.error_mode):
                raise UnknownConfigType(
                    f"config file: {self.filename} is not supported."
                )
        return self.config_data



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "openafs", "version": "1.6.12", "version_strings": ["OpenAFS 1.6.12"]}
]
package_test_data = [
    {
        "url": "http://rpmfind.net/linux/mageia/distrib/5/x86_64/media/core/updates/",
        "package_name": "openafs-client-1.6.13-1.mga5.x86_64.rpm",
        "product": "openafs",
        "version": "1.6.13",
    },
    {
        "url": "http://archive.ubuntu.com/ubuntu/pool/universe/o/openafs/",
        "package_name": "openafs-client_1.6.15-1ubuntu1_amd64.deb",
        "product": "openafs",
        "version": "1.6.15",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "busybox", "version": "1.18.3", "version_strings": ["BusyBox v1.18.3"]}
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/busybox/1.28.2/1.fc29/x86_64/",
        "package_name": "busybox-1.28.2-1.fc29.x86_64.rpm",
        "product": "busybox",
        "version": "1.28.2",
    },
    {
        "url": "http://archive.ubuntu.com/ubuntu/pool/universe/b/busybox/",
        "package_name": "busybox_1.18.5-1ubuntu4_amd64.deb",
        "product": "busybox",
        "version": "1.18.5",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {"product": "dnsmasq", "version": "2.79", "version_strings": ["dnsmasq-2.79"]},
    {"product": "dnsmasq", "version": "2.84", "version_strings": ["dnsmasq-2.84"]},
]

package_test_data = [
    {
        "url": "http://mirror.centos.org/centos/8/AppStream/aarch64/os/Packages/",
        "package_name": "dnsmasq-2.79-13.el8_3.1.aarch64.rpm",
        "product": "dnsmasq",
        "version": "2.79",
    },
    {
        "url": "https://download-ib01.fedoraproject.org/pub/fedora/linux/development/rawhide/Everything/aarch64/os/Packages/d/",
        "package_name": "dnsmasq-2.84-1.fc34.aarch64.rpm",
        "product": "dnsmasq",
        "version": "2.84",
    },
]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later


"""
CVE checker for logrotate

References: https://www.cvedetails.com/product/20629/?q=Logrotate

"""
from cve_bin_tool.checkers import Checker


class LogrotateChecker(Checker):
    CONTAINS_PATTERNS = [
        r"Set \"su\" directive in config file to tell logrotate which user/group should be used for rotation",
        r"WARNING: logrotate in debug mode does nothing except printing debug messages!  Consider using verbose mode \(-v\) instead if this is not what you want.",
    ]
    FILENAME_PATTERNS = [r"logrotate"]
    VERSION_PATTERNS = [
        r"logrotate ([0-9]+\.[0-9]+\.[0-9]+) - Copyright \(C\) 1995-2001 Red Hat, Inc."
    ]
    VENDOR_PRODUCT = [("gentoo", "logrotate")]



# Copyright (C) 2021 Intel Corporation
# SPDX-License-Identifier: GPL-3.0-or-later

mapping_test_data = [
    {
        "product": "postgresql",
        "version": "9.4.3",
        "version_strings": ["PostgreSQL 9.4.3"],
    }
]
package_test_data = [
    {
        "url": "https://kojipkgs.fedoraproject.org/packages/postgresql/9.4.4/1.fc22/x86_64/",
        "package_name": "postgresql-9.4.4-1.fc22.x86_64.rpm",
        "product": "postgresql",
        "version": "9.4.4",
    }
]
